@@ -129,9 +129,6 @@ void		 gs_app_set_id_kind		(GsApp		*app,
 AsAppState	 gs_app_get_state		(GsApp		*app);
 void		 gs_app_set_state		(GsApp		*app,
 						 AsAppState	 state);
-guint		 gs_app_get_progress		(GsApp		*app);
-void		 gs_app_set_progress		(GsApp		*app,
-						 guint		 percentage);
 const gchar	*gs_app_get_name		(GsApp		*app);
 void		 gs_app_set_name		(GsApp		*app,
 						 GsAppQuality	 quality,
@@ -197,12 +194,9 @@ void		 gs_app_set_management_plugin	(GsApp		*app,
 GdkPixbuf	*gs_app_get_pixbuf		(GsApp		*app);
 void		 gs_app_set_pixbuf		(GsApp		*app,
 						 GdkPixbuf	*pixbuf);
-const gchar	*gs_app_get_icon		(GsApp		*app);
+AsIcon		*gs_app_get_icon		(GsApp		*app);
 void		 gs_app_set_icon		(GsApp		*app,
-						 const gchar	*icon);
-const gchar	*gs_app_get_icon_path		(GsApp		*app);
-void		 gs_app_set_icon_path		(GsApp		*app,
-						 const gchar	*icon_path);
+						 AsIcon		*icon);
 gboolean	 gs_app_load_icon		(GsApp		*app,
 						 gint		 scale,
 						 GError		**error);
