@@ -24,6 +24,7 @@
 #include <string.h>
 #include <glib/gi18n.h>
 
+#include "gs-cleanup.h"
 #include "gs-shell-search.h"
 #include "gs-shell.h"
 #include "gs-app.h"
@@ -54,11 +55,10 @@ G_DEFINE_TYPE_WITH_PRIVATE (GsShellSearch, gs_shell_search, GTK_TYPE_BIN)
 
 static void
 gs_shell_search_app_row_activated_cb (GtkListBox *list_box,
-                                      GtkListBoxRow *row,
-                                      GsShellSearch *shell_search)
+				      GtkListBoxRow *row,
+				      GsShellSearch *shell_search)
 {
 	GsApp *app;
-
 	app = gs_app_row_get_app (GS_APP_ROW (row));
 	gs_shell_show_app (shell_search->priv->shell, app);
 }
@@ -73,10 +73,10 @@ gs_shell_search_app_installed_cb (GObject *source,
 				  GAsyncResult *res,
 				  gpointer user_data)
 {
-	GError *error = NULL;
 	GsPluginLoader *plugin_loader = GS_PLUGIN_LOADER (source);
 	GsShellSearchHelper *helper = (GsShellSearchHelper *) user_data;
 	gboolean ret;
+	_cleanup_error_free_ GError *error = NULL;
 
 	ret = gs_plugin_loader_app_action_finish (plugin_loader,
 						  res,
@@ -89,7 +89,6 @@ gs_shell_search_app_installed_cb (GObject *source,
 					    gs_shell_get_window (helper->shell_search->priv->shell),
 					    GS_PLUGIN_LOADER_ACTION_INSTALL,
 					    error);
-		g_error_free (error);
 	} else {
 		/* only show this if the window is not active */
 		if (!gs_shell_is_active (helper->shell_search->priv->shell))
@@ -109,22 +108,20 @@ gs_shell_search_app_removed_cb (GObject *source,
 				GAsyncResult *res,
 				gpointer user_data)
 {
-	GError *error = NULL;
 	GsPluginLoader *plugin_loader = GS_PLUGIN_LOADER (source);
 	GsShellSearchHelper *helper = (GsShellSearchHelper *) user_data;
 	gboolean ret;
+	_cleanup_error_free_ GError *error = NULL;
 
 	ret = gs_plugin_loader_app_action_finish (plugin_loader,
 						  res,
 						  &error);
 	if (!ret) {
-		g_warning ("failed to remove: %s",
-			   error->message);
+		g_warning ("failed to remove: %s", error->message);
 		gs_app_notify_failed_modal (helper->app,
 					    gs_shell_get_window (helper->shell_search->priv->shell),
 					    GS_PLUGIN_LOADER_ACTION_REMOVE,
 					    error);
-		g_error_free (error);
 	}
 	gs_shell_search_reload (helper->shell_search);
 	g_object_unref (helper->app);
@@ -139,9 +136,9 @@ static void
 gs_shell_search_app_remove (GsShellSearch *shell_search, GsApp *app)
 {
 	GsShellSearchPrivate *priv = shell_search->priv;
-	GString *markup;
 	GtkResponseType response;
 	GtkWidget *dialog;
+	_cleanup_string_free_ GString *markup = NULL;
 
 	markup = g_string_new ("");
 	g_string_append_printf (markup,
@@ -177,7 +174,6 @@ gs_shell_search_app_remove (GsShellSearch *shell_search, GsApp *app)
 						   gs_shell_search_app_removed_cb,
 						   helper);
 	}
-	g_string_free (markup, TRUE);
 	gtk_widget_destroy (dialog);
 }
 
@@ -206,15 +202,37 @@ gs_shell_search_app_install (GsShellSearch *shell_search, GsApp *app)
 static void
 gs_shell_search_show_missing_url (GsApp *app)
 {
-	GError *error = NULL;
 	const gchar *url;
-	gboolean ret;
+	_cleanup_error_free_ GError *error = NULL;
 
 	url = gs_app_get_url (app, AS_URL_KIND_MISSING);
-	ret = gtk_show_uri (NULL, url, GDK_CURRENT_TIME, &error);
-	if (!ret) {
+	if (!gtk_show_uri (NULL, url, GDK_CURRENT_TIME, &error))
 		g_warning ("spawn of '%s' failed", url);
-		g_error_free (error);
+}
+
+/**
+ * gs_shell_search_install_unavailable_app:
+ **/
+static void
+gs_shell_search_install_unavailable_app (GsShellSearch *shell_search, GsApp *app)
+{
+	GsShellSearchPrivate *priv = shell_search->priv;
+	GtkResponseType response;
+	GsShellSearchHelper *helper;
+
+	/* get confirmation */
+	response = gs_app_notify_unavailable (app, gs_shell_get_window (priv->shell));
+	if (response == GTK_RESPONSE_OK) {
+		g_debug ("installing %s", gs_app_get_id (app));
+		helper = g_new0 (GsShellSearchHelper, 1);
+		helper->shell_search = g_object_ref (shell_search);
+		helper->app = g_object_ref (app);
+		gs_plugin_loader_app_action_async (priv->plugin_loader,
+						   app,
+						   GS_PLUGIN_LOADER_ACTION_INSTALL,
+						   priv->cancellable,
+						   gs_shell_search_app_installed_cb,
+						   helper);
 	}
 }
 
@@ -223,7 +241,7 @@ gs_shell_search_show_missing_url (GsApp *app)
  **/
 static void
 gs_shell_search_app_row_clicked_cb (GsAppRow *app_row,
-                                    GsShellSearch *shell_search)
+				    GsShellSearch *shell_search)
 {
 	GsApp *app;
 	app = gs_app_row_get_app (app_row);
@@ -231,8 +249,13 @@ gs_shell_search_app_row_clicked_cb (GsAppRow *app_row,
 		gs_shell_search_app_install (shell_search, app);
 	else if (gs_app_get_state (app) == AS_APP_STATE_INSTALLED)
 		gs_shell_search_app_remove (shell_search, app);
-	else if (gs_app_get_state (app) == AS_APP_STATE_UNAVAILABLE)
+	else if (gs_app_get_state (app) == AS_APP_STATE_UNAVAILABLE) {
+		if (gs_app_get_url (app, AS_URL_KIND_MISSING) == NULL) {
+			gs_shell_search_install_unavailable_app (shell_search, app);
+			return;
+		}
 		gs_shell_search_show_missing_url (app);
+	}
 }
 
 /**
@@ -243,7 +266,6 @@ gs_shell_search_get_search_cb (GObject *source_object,
 				     GAsyncResult *res,
 				     gpointer user_data)
 {
-	GError *error = NULL;
 	GList *l;
 	GList *list;
 	GsApp *app;
@@ -251,29 +273,24 @@ gs_shell_search_get_search_cb (GObject *source_object,
 	GsShellSearchPrivate *priv = shell_search->priv;
 	GsPluginLoader *plugin_loader = GS_PLUGIN_LOADER (source_object);
 	GtkWidget *app_row;
+	_cleanup_error_free_ GError *error = NULL;
 
 	list = gs_plugin_loader_search_finish (plugin_loader, res, &error);
 	if (list == NULL) {
-		if (g_error_matches (error,
-		                     G_IO_ERROR,
-		                     G_IO_ERROR_CANCELLED)) {
+		if (g_error_matches (error, G_IO_ERROR, G_IO_ERROR_CANCELLED)) {
 			g_debug ("search cancelled");
-			g_error_free (error);
-			goto out;
+			return;
 		}
-
 		if (g_error_matches (error,
 				     GS_PLUGIN_LOADER_ERROR,
 				     GS_PLUGIN_LOADER_ERROR_NO_RESULTS)) {
 			g_debug ("no search results to show");
 		} else {
-			g_warning ("failed to get search apps: %s",
-				   error->message);
+			g_warning ("failed to get search apps: %s", error->message);
 		}
-		g_error_free (error);
 		gs_stop_spinner (GTK_SPINNER (priv->spinner_search));
 		gtk_stack_set_visible_child_name (GTK_STACK (priv->stack_search), "no-results");
-		goto out;
+		return;
 	}
 
 	gs_stop_spinner (GTK_SPINNER (priv->spinner_search));
@@ -287,8 +304,8 @@ gs_shell_search_get_search_cb (GObject *source_object,
 		gs_app_row_set_app (GS_APP_ROW (app_row), app);
 		gtk_container_add (GTK_CONTAINER (priv->list_box_search), app_row);
 		gs_app_row_set_size_groups (GS_APP_ROW (app_row),
-		                            priv->sizegroup_image,
-		                            priv->sizegroup_name);
+					    priv->sizegroup_image,
+					    priv->sizegroup_name);
 		gtk_widget_show (app_row);
 	}
 
@@ -296,8 +313,6 @@ gs_shell_search_get_search_cb (GObject *source_object,
 		gs_shell_show_details (priv->shell, priv->appid_to_show);
 		g_clear_pointer (&priv->appid_to_show, g_free);
 	}
-
-out: ;
 }
 
 /**
@@ -382,11 +397,11 @@ gs_shell_search_switch_to (GsShellSearch *shell_search, const gchar *value, gboo
 	widget = GTK_WIDGET (gtk_builder_get_object (priv->builder, "search_bar"));
 	gtk_widget_show (widget);
 
-        if (scroll_up) {
-                GtkAdjustment *adj;
-                adj = gtk_scrolled_window_get_vadjustment (GTK_SCROLLED_WINDOW (priv->scrolledwindow_search));
-                gtk_adjustment_set_value (adj, gtk_adjustment_get_lower (adj));
-        }
+	if (scroll_up) {
+		GtkAdjustment *adj;
+		adj = gtk_scrolled_window_get_vadjustment (GTK_SCROLLED_WINDOW (priv->scrolledwindow_search));
+		gtk_adjustment_set_value (adj, gtk_adjustment_get_lower (adj));
+	}
 
 	g_free (priv->value);
 	priv->value = g_strdup (value);
@@ -465,17 +480,11 @@ gs_shell_search_sort_func (GtkListBoxRow *a,
 {
 	GsApp *a1 = gs_app_row_get_app (GS_APP_ROW (a));
 	GsApp *a2 = gs_app_row_get_app (GS_APP_ROW (b));
-	gchar *key1 = gs_shell_search_get_app_sort_key (a1);
-	gchar *key2 = gs_shell_search_get_app_sort_key (a2);
-	gint retval;
+	_cleanup_free_ gchar *key1 = gs_shell_search_get_app_sort_key (a1);
+	_cleanup_free_ gchar *key2 = gs_shell_search_get_app_sort_key (a2);
 
 	/* compare the keys according to the algorithm above */
-	retval = g_strcmp0 (key2, key1);
-
-	g_free (key1);
-	g_free (key2);
-
-	return retval;
+	return g_strcmp0 (key2, key1);
 }
 
 /**
