@@ -25,6 +25,7 @@
 #include <string.h>
 #include <glib/gi18n.h>
 
+#include "gs-cleanup.h"
 #include "gs-utils.h"
 #include "gs-shell.h"
 #include "gs-shell-details.h"
@@ -35,7 +36,6 @@
 #include "gs-shell-category.h"
 #include "gs-sources-dialog.h"
 #include "gs-update-dialog.h"
-#include "gs-offline-updates.h"
 
 static const gchar *page_name[] = {
 	"overview",
@@ -171,8 +171,6 @@ gs_shell_change_mode (GsShell *shell,
 
 	widget = GTK_WIDGET (gtk_builder_get_object (priv->builder, "button_updates"));
 	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (widget), mode == GS_SHELL_MODE_UPDATES);
-	gtk_widget_set_visible (widget, !gs_updates_are_managed() || mode == GS_SHELL_MODE_UPDATES);
-
 	priv->ignore_primary_buttons = FALSE;
 
 	/* switch page */
@@ -231,14 +229,14 @@ save_back_entry (GsShell *shell)
 {
 	GsShellPrivate *priv = shell->priv;
 	BackEntry *entry;
+	GtkWidget *focus;
 
 	entry = g_new0 (BackEntry, 1);
 	entry->mode = priv->mode;
 
-	entry->focus = gtk_window_get_focus (priv->main_window);
-	if (entry->focus != NULL)
-		g_object_add_weak_pointer (G_OBJECT (entry->focus),
-					   (gpointer *) &entry->focus);
+	focus = gtk_window_get_focus (priv->main_window);
+	if (focus != NULL)
+		entry->focus = g_object_ref (focus);
 
 	if (priv->mode == GS_SHELL_MODE_CATEGORY) {
 		entry->category = gs_shell_category_get_category (priv->shell_category);
@@ -255,9 +253,7 @@ save_back_entry (GsShell *shell)
 static void
 free_back_entry (BackEntry *entry)
 {
-	if (entry->focus != NULL)
-		g_object_remove_weak_pointer (G_OBJECT (entry->focus),
-		                              (gpointer *) &entry->focus);
+	g_clear_object (&entry->focus);
 	g_clear_object (&entry->category);
 	g_clear_object (&entry->app);
 	g_free (entry);
@@ -278,9 +274,8 @@ gs_shell_back_button_cb (GtkWidget *widget, GsShell *shell)
 
 	gs_shell_change_mode (shell, entry->mode, entry->app, entry->category, FALSE);
 
-	if (entry->focus) {
+	if (entry->focus != NULL)
 		gtk_widget_grab_focus (entry->focus);
-	}
 
 	free_back_entry (entry);
 }
@@ -377,7 +372,8 @@ window_keypress_handler (GtkWidget *window, GdkEvent *event, GsShell *shell)
 	gboolean preedit_changed;
 	guint preedit_change_id;
 	gboolean res;
-	gchar *old_text, *new_text;
+	_cleanup_free_ gchar *old_text = NULL;
+	_cleanup_free_ gchar *new_text = NULL;
 
 	if (gs_shell_get_mode (shell) != GS_SHELL_MODE_OVERVIEW &&
 	    gs_shell_get_mode (shell) != GS_SHELL_MODE_SEARCH)
@@ -406,9 +402,6 @@ window_keypress_handler (GtkWidget *window, GdkEvent *event, GsShell *shell)
 	    preedit_changed)
 		handled = GDK_EVENT_STOP;
 
-	g_free (old_text);
-	g_free (new_text);
-
 	/* We set "editable" so the text in the  entry won't get selected on focus */
 	g_object_set (entry, "editable", FALSE, NULL);
 	gtk_widget_grab_focus (entry);
@@ -517,29 +510,6 @@ gs_shell_main_window_mapped_cb (GtkWidget *widget, GsShell *shell)
 				    gtk_widget_get_scale_factor (widget));
 }
 
-static void
-on_permission_changed (GPermission *permission,
-                       GParamSpec  *pspec,
-                       gpointer     data)
-{
-        GsShell *shell = data;
-	GsShellPrivate *priv = shell->priv;
-	GtkWidget *widget;
-
-	widget = GTK_WIDGET (gtk_builder_get_object (priv->builder, "button_updates"));
-	gtk_widget_set_visible (widget, !gs_updates_are_managed() || priv->mode == GS_SHELL_MODE_UPDATES);
-}
-
-static void
-gs_shell_monitor_permission (GsShell *shell)
-{
-        GPermission *permission;
-
-        permission = gs_offline_updates_permission_get ();
-        g_signal_connect (permission, "notify",
-                          G_CALLBACK (on_permission_changed), shell);
-}
-
 /**
  * gs_shell_setup:
  */
@@ -556,8 +526,6 @@ gs_shell_setup (GsShell *shell, GsPluginLoader *plugin_loader, GCancellable *can
 			  G_CALLBACK (gs_shell_updates_changed_cb), shell);
 	priv->cancellable = g_object_ref (cancellable);
 
-	gs_shell_monitor_permission (shell);
-
 	/* get UI */
 	priv->builder = gtk_builder_new_from_resource ("/org/gnome/Software/gnome-software.ui");
 	priv->main_window = GTK_WINDOW (gtk_builder_get_object (priv->builder, "window_software"));
@@ -569,7 +537,7 @@ gs_shell_setup (GsShell *shell, GsPluginLoader *plugin_loader, GCancellable *can
 					   GS_DATA G_DIR_SEPARATOR_S "icons");
 
 	g_signal_connect (priv->main_window, "delete-event",
-	                  G_CALLBACK (main_window_closed_cb), shell);
+			  G_CALLBACK (main_window_closed_cb), shell);
 
 	/* fix up the header bar */
 	widget = GTK_WIDGET (gtk_builder_get_object (priv->builder, "header"));
@@ -696,13 +664,13 @@ gs_shell_get_mode_string (GsShell *shell)
 
 static void
 gs_shell_get_installed_updates_cb (GsPluginLoader *plugin_loader,
-                                   GAsyncResult *res,
-                                   GsShell *shell)
+				   GAsyncResult *res,
+				   GsShell *shell)
 {
 	GsShellPrivate *priv = shell->priv;
-	GError *error = NULL;
 	GList *list;
 	GtkWidget *dialog;
+	_cleanup_error_free_ GError *error = NULL;
 
 	/* get the results */
 	list = gs_plugin_loader_get_updates_finish (plugin_loader, res, &error);
@@ -712,13 +680,12 @@ gs_shell_get_installed_updates_cb (GsPluginLoader *plugin_loader,
 				     GS_PLUGIN_LOADER_ERROR_NO_RESULTS)) {
 			g_debug ("no updates to show");
 		} else if (g_error_matches (error,
-		                            G_IO_ERROR,
-		                            G_IO_ERROR_CANCELLED)) {
+					    G_IO_ERROR,
+					    G_IO_ERROR_CANCELLED)) {
 			g_debug ("get updates cancelled");
 		} else {
 			g_warning ("failed to get updates: %s", error->message);
 		}
-		g_error_free (error);
 		goto out;
 	}
 
@@ -740,15 +707,15 @@ gs_shell_show_installed_updates (GsShell *shell)
 	guint64 refine_flags;
 
 	refine_flags = GS_PLUGIN_REFINE_FLAGS_DEFAULT |
-	               GS_PLUGIN_REFINE_FLAGS_REQUIRE_UPDATE_DETAILS |
-	               GS_PLUGIN_REFINE_FLAGS_REQUIRE_VERSION |
-	               GS_PLUGIN_REFINE_FLAGS_USE_HISTORY;
+		       GS_PLUGIN_REFINE_FLAGS_REQUIRE_UPDATE_DETAILS |
+		       GS_PLUGIN_REFINE_FLAGS_REQUIRE_VERSION |
+		       GS_PLUGIN_REFINE_FLAGS_USE_HISTORY;
 
 	gs_plugin_loader_get_updates_async (priv->plugin_loader,
-	                                    refine_flags,
-	                                    priv->cancellable,
-	                                    (GAsyncReadyCallback) gs_shell_get_installed_updates_cb,
-	                                    shell);
+					    refine_flags,
+					    priv->cancellable,
+					    (GAsyncReadyCallback) gs_shell_get_installed_updates_cb,
+					    shell);
 }
 
 void
@@ -807,7 +774,7 @@ gs_shell_show_search_result (GsShell *shell, const gchar *id, const gchar *searc
 	widget = GTK_WIDGET (gtk_builder_get_object (priv->builder, "entry_search"));
 
 	/* ignore next "search-changed" signal to avoid getting a callback
-         * after 150 ms and messing up the state */
+	 * after 150 ms and messing up the state */
 	priv->ignore_next_search_changed_signal = TRUE;
 	gtk_entry_set_text (GTK_ENTRY (widget), search);
 
@@ -818,11 +785,9 @@ gs_shell_show_search_result (GsShell *shell, const gchar *id, const gchar *searc
 void
 gs_shell_show_details (GsShell *shell, const gchar *id)
 {
-	GsApp *app;
-
+	_cleanup_object_unref_ GsApp *app = NULL;
 	app = gs_app_new (id);
 	gs_shell_show_app (shell, app);
-	g_object_unref (app);
 }
 
 /**
