@@ -51,7 +51,7 @@ gs_plugin_get_deps (GsPlugin *plugin)
 /**
  * gs_plugin_refine_app_category:
  */
-static void
+static gboolean
 gs_plugin_refine_app_category (GsPlugin *plugin,
 			       GsApp *app,
 			       const MenuSpecData *cat)
@@ -71,18 +71,15 @@ gs_plugin_refine_app_category (GsPlugin *plugin,
 			continue;
 		ret = gs_app_has_category (app, tmp + 2);
 		if (ret) {
-			tmp = g_strdup_printf ("%s → %s",
+			_cleanup_free_ gchar *str = NULL;
+			str = g_strdup_printf ("%s → %s",
 					       gettext (cat->text),
 					       gettext (msdata[i].text));
-			gs_app_set_menu_path (app, tmp);
-			g_free (tmp);
+			gs_app_set_menu_path (app, str);
 			break;
 		}
 	}
-
-	/* if we've failed to find a matching sub-level category, just show the top-level */
-	if (!ret)
-		gs_app_set_menu_path (app, gettext (cat->text));
+	return ret;
 }
 
 /**
@@ -104,8 +101,8 @@ gs_plugin_refine_app (GsPlugin *plugin, GsApp *app)
 			continue;
 		ret = gs_app_has_category (app, msdata[i].path);
 		if (ret) {
-			gs_plugin_refine_app_category (plugin, app,
-			                               &msdata[i]);
+			ret = gs_plugin_refine_app_category (plugin, app,
+							     &msdata[i]);
 			break;
 		}
 	}
@@ -124,22 +121,19 @@ gs_plugin_refine (GsPlugin *plugin,
 {
 	GList *l;
 	GsApp *app;
-	gboolean ret;
 
 	/* nothing to do here */
 	if ((flags & GS_PLUGIN_REFINE_FLAGS_REQUIRE_MENU_PATH) == 0)
-		goto out;
+		return TRUE;
 
 	for (l = *list; l != NULL; l = l->next) {
 		app = GS_APP (l->data);
 		if (gs_app_get_menu_path (app) == NULL) {
-			ret = gs_plugin_refine_app (plugin, app);
-			if (!ret) {
+			if (!gs_plugin_refine_app (plugin, app)) {
 				/* don't keep searching for this */
 				gs_app_set_menu_path (app, "");
 			}
 		}
 	}
-out:
 	return TRUE;
 }
