@@ -106,9 +106,8 @@ gs_plugin_add_installed (GsPlugin *plugin,
 			 GCancellable *cancellable,
 			 GError **error)
 {
-	gboolean ret = TRUE;
 	PkBitfield filter;
-	PkResults *results;
+	_cleanup_object_unref_ PkResults *results = NULL;
 
 	/* update UI as this might take some time */
 	gs_plugin_status_update (plugin, NULL, GS_PLUGIN_STATUS_WAITING);
@@ -125,19 +124,11 @@ gs_plugin_add_installed (GsPlugin *plugin,
 					  cancellable,
 					  gs_plugin_packagekit_progress_cb, plugin,
 					  error);
-	if (results == NULL) {
-		ret = FALSE;
-		goto out;
-	}
+	if (results == NULL)
+		return FALSE;
 
 	/* add results */
-	ret = gs_plugin_packagekit_add_results (plugin, list, results, error);
-	if (!ret)
-		goto out;
-out:
-	if (results != NULL)
-		g_object_unref (results);
-	return ret;
+	return gs_plugin_packagekit_add_results (plugin, list, results, error);
 }
 
 /**
@@ -154,10 +145,9 @@ gs_plugin_add_sources_related (GsPlugin *plugin,
 	GsApp *app;
 	GsApp *app_tmp;
 	PkBitfield filter;
-	PkResults *results = NULL;
 	const gchar *id;
 	gboolean ret = TRUE;
-	gchar **split;
+	_cleanup_object_unref_ PkResults *results = NULL;
 
 	gs_profile_start (plugin->profile, "packagekit::add-sources-related");
 	filter = pk_bitfield_from_enums (PK_FILTER_ENUM_INSTALLED,
@@ -181,6 +171,7 @@ gs_plugin_add_sources_related (GsPlugin *plugin,
 	if (!ret)
 		goto out;
 	for (l = installed; l != NULL; l = l->next) {
+		_cleanup_strv_free_ gchar **split = NULL;
 		app = GS_APP (l->data);
 		split = pk_package_id_split (gs_app_get_source_id_default (app));
 		if (g_str_has_prefix (split[PK_PACKAGE_ID_DATA], "installed:")) {
@@ -192,13 +183,10 @@ gs_plugin_add_sources_related (GsPlugin *plugin,
 				gs_app_add_related (app_tmp, app);
 			}
 		}
-		g_strfreev (split);
 	}
 out:
 	gs_profile_stop (plugin->profile, "packagekit::add-sources-related");
 	gs_plugin_list_free (installed);
-	if (results != NULL)
-		g_object_unref (results);
 	return ret;
 }
 
@@ -211,15 +199,13 @@ gs_plugin_add_sources (GsPlugin *plugin,
 		       GCancellable *cancellable,
 		       GError **error)
 {
-	GPtrArray *array = NULL;
-	GsApp *app;
 	PkBitfield filter;
 	PkRepoDetail *rd;
-	PkResults *results;
 	const gchar *id;
-	gboolean ret = TRUE;
 	guint i;
-	GHashTable *hash = NULL;
+	_cleanup_hashtable_unref_ GHashTable *hash = NULL;
+	_cleanup_object_unref_ PkResults *results = NULL;
+	_cleanup_ptrarray_unref_ GPtrArray *array = NULL;
 
 	/* ask PK for the repo details */
 	filter = pk_bitfield_from_enums (PK_FILTER_ENUM_NOT_SOURCE,
@@ -231,15 +217,14 @@ gs_plugin_add_sources (GsPlugin *plugin,
 					   cancellable,
 					   gs_plugin_packagekit_progress_cb, plugin,
 					   error);
-	if (results == NULL) {
-		ret = FALSE;
-		goto out;
-	}
+	if (results == NULL)
+		return FALSE;
 	hash = g_hash_table_new_full (g_str_hash, g_str_equal, g_free, NULL);
 	array = pk_results_get_repo_detail_array (results);
-	for (i = 0; i < array->len; i++) {
-		rd = g_ptr_array_index (array, i);
 #if PK_CHECK_VERSION(0,9,1)
+	for (i = 0; i < array->len; i++) {
+		_cleanup_object_unref_ GsApp *app = NULL;
+		rd = g_ptr_array_index (array, i);
 		id = pk_repo_detail_get_id (rd);
 		app = gs_app_new (id);
 		gs_app_set_management_plugin (app, "PackageKit");
@@ -255,23 +240,34 @@ gs_plugin_add_sources (GsPlugin *plugin,
 		g_hash_table_insert (hash,
 				     g_strdup (id),
 				     (gpointer) app);
-		g_object_unref (app);
-#endif
 	}
+#endif
 
 	/* get every application on the system and add it as a related package
 	 * if it matches */
-	ret = gs_plugin_add_sources_related (plugin, hash, cancellable, error);
-	if (!ret)
-		goto out;
-out:
-	if (hash != NULL)
-		g_hash_table_unref (hash);
-	if (array != NULL)
-		g_ptr_array_unref (array);
-	if (results != NULL)
-		g_object_unref (results);
-	return ret;
+	return gs_plugin_add_sources_related (plugin, hash, cancellable, error);
+}
+
+/**
+ * gs_plugin_app_source_enable:
+ */
+static gboolean
+gs_plugin_app_source_enable (GsPlugin *plugin,
+			     GsApp *app,
+			     GCancellable *cancellable,
+			     GError **error)
+{
+	_cleanup_object_unref_ PkResults *results = NULL;
+
+	/* do sync call */
+	gs_plugin_status_update (plugin, NULL, GS_PLUGIN_STATUS_WAITING);
+	results = pk_client_repo_enable (PK_CLIENT (plugin->priv->task),
+					 gs_app_get_origin (app),
+					 TRUE,
+					 cancellable,
+					 gs_plugin_packagekit_progress_cb, plugin,
+					 error);
+	return results != NULL;
 }
 
 /**
@@ -284,18 +280,53 @@ gs_plugin_app_install (GsPlugin *plugin,
 		       GError **error)
 {
 	GPtrArray *addons;
-	GPtrArray *array_package_ids = NULL;
 	GPtrArray *source_ids;
-	PkError *error_code = NULL;
-	PkResults *results = NULL;
 	const gchar *package_id;
-	gboolean ret = TRUE;
-	gchar **package_ids = NULL;
 	guint i, j;
+	_cleanup_object_unref_ PkError *error_code = NULL;
+	_cleanup_object_unref_ PkResults *results = NULL;
+	_cleanup_ptrarray_unref_ GPtrArray *array_package_ids = NULL;
+	_cleanup_strv_free_ gchar **package_ids = NULL;
 
 	/* only process this app if was created by this plugin */
 	if (g_strcmp0 (gs_app_get_management_plugin (app), "PackageKit") != 0)
-		goto out;
+		return TRUE;
+
+	/* we enable the repo */
+	if (gs_app_get_state (app) == AS_APP_STATE_UNAVAILABLE) {
+
+		/* get everything up front we need */
+		source_ids = gs_app_get_source_ids (app);
+		package_ids = g_new0 (gchar *, 2);
+		package_ids[0] = g_strdup (g_ptr_array_index (source_ids, 0));
+
+		/* enable the source */
+		if (!gs_plugin_app_source_enable (plugin, app, cancellable, error))
+			return FALSE;
+
+		/* FIXME: this is a hack, to allow PK time to re-initialize
+		 * everything in order to match an actual result. The root cause
+		 * is probably some kind of hard-to-debug race in the daemon. */
+		g_usleep (G_USEC_PER_SEC * 3);
+
+		/* actually install the package */
+		gs_app_set_state (app, AS_APP_STATE_AVAILABLE);
+		gs_app_set_state (app, AS_APP_STATE_INSTALLING);
+		results = pk_task_install_packages_sync (plugin->priv->task,
+							 package_ids,
+							 cancellable,
+							 gs_plugin_packagekit_progress_cb, plugin,
+							 error);
+		if (results == NULL) {
+			gs_app_set_state (app, AS_APP_STATE_AVAILABLE);
+			return FALSE;
+		}
+
+		/* no longer valid */
+		gs_app_clear_source_ids (app);
+		gs_app_set_state (app, AS_APP_STATE_INSTALLED);
+		return TRUE;
+	}
 
 	/* get the list of available package ids to install */
 	switch (gs_app_get_state (app)) {
@@ -303,12 +334,11 @@ gs_plugin_app_install (GsPlugin *plugin,
 	case AS_APP_STATE_UPDATABLE:
 		source_ids = gs_app_get_source_ids (app);
 		if (source_ids->len == 0) {
-			ret = FALSE;
 			g_set_error_literal (error,
 					     GS_PLUGIN_ERROR,
 					     GS_PLUGIN_ERROR_NOT_SUPPORTED,
 					     "installing not available");
-			goto out;
+			return FALSE;
 		}
 		array_package_ids = g_ptr_array_new_with_free_func (g_free);
 		for (i = 0; i < source_ids->len; i++) {
@@ -336,12 +366,11 @@ gs_plugin_app_install (GsPlugin *plugin,
 		g_ptr_array_add (array_package_ids, NULL);
 
 		if (array_package_ids->len == 0) {
-			ret = FALSE;
 			g_set_error_literal (error,
 					     GS_PLUGIN_ERROR,
 					     GS_PLUGIN_ERROR_NOT_SUPPORTED,
 					     "no packages to install");
-			goto out;
+			return FALSE;
 		}
 		gs_app_set_state (app, AS_APP_STATE_INSTALLING);
 		addons = gs_app_get_addons (app);
@@ -355,20 +384,17 @@ gs_plugin_app_install (GsPlugin *plugin,
 							 cancellable,
 							 gs_plugin_packagekit_progress_cb, plugin,
 							 error);
-		if (results == NULL) {
-			ret = FALSE;
-			goto out;
-		}
+		if (results == NULL)
+			return FALSE;
 		break;
 	case AS_APP_STATE_AVAILABLE_LOCAL:
 		package_id = gs_app_get_metadata_item (app, "PackageKit::local-filename");
 		if (package_id == NULL) {
-			ret = FALSE;
 			g_set_error_literal (error,
 					     GS_PLUGIN_ERROR,
 					     GS_PLUGIN_ERROR_NOT_SUPPORTED,
 					     "local package, but no filename");
-			goto out;
+			return FALSE;
 		}
 		package_ids = g_strsplit (package_id, "\t", -1);
 		gs_app_set_state (app, AS_APP_STATE_INSTALLING);
@@ -377,19 +403,16 @@ gs_plugin_app_install (GsPlugin *plugin,
 						      cancellable,
 						      gs_plugin_packagekit_progress_cb, plugin,
 						      error);
-		if (results == NULL) {
-			ret = FALSE;
-			goto out;
-		}
+		if (results == NULL)
+			return FALSE;
 		break;
 	default:
-		ret = FALSE;
 		g_set_error (error,
 			     GS_PLUGIN_ERROR,
 			     GS_PLUGIN_ERROR_FAILED,
 			     "do not know how to install app in state %s",
 			     as_app_state_to_string (gs_app_get_state (app)));
-		goto out;
+		return FALSE;
 	}
 
 	/* no longer valid */
@@ -398,24 +421,15 @@ gs_plugin_app_install (GsPlugin *plugin,
 	/* check error code */
 	error_code = pk_results_get_error_code (results);
 	if (error_code != NULL) {
-		ret = FALSE;
 		g_set_error (error,
 			     GS_PLUGIN_ERROR,
 			     GS_PLUGIN_ERROR_FAILED,
 			     "failed to install package: %s, %s",
 			     pk_error_enum_to_string (pk_error_get_code (error_code)),
 			     pk_error_get_details (error_code));
-		goto out;
+		return FALSE;
 	}
-out:
-	g_strfreev (package_ids);
-	if (error_code != NULL)
-		g_object_unref (error_code);
-	if (array_package_ids != NULL)
-		g_ptr_array_unref (array_package_ids);
-	if (results != NULL)
-		g_object_unref (results);
-	return ret;
+	return TRUE;
 }
 
 /**
@@ -427,8 +441,7 @@ gs_plugin_app_source_disable (GsPlugin *plugin,
 			      GCancellable *cancellable,
 			      GError **error)
 {
-	gboolean ret = TRUE;
-	PkResults *results;
+	_cleanup_object_unref_ PkResults *results = NULL;
 
 	/* do sync call */
 	gs_plugin_status_update (plugin, NULL, GS_PLUGIN_STATUS_WAITING);
@@ -438,14 +451,7 @@ gs_plugin_app_source_disable (GsPlugin *plugin,
 					 cancellable,
 					 gs_plugin_packagekit_progress_cb, plugin,
 					 error);
-	if (results == NULL) {
-		ret = FALSE;
-		goto out;
-	}
-out:
-	if (results != NULL)
-		g_object_unref (results);
-	return ret;
+	return results != NULL;
 }
 
 /**
@@ -457,10 +463,9 @@ gs_plugin_app_source_remove (GsPlugin *plugin,
 			     GCancellable *cancellable,
 			     GError **error)
 {
-	gboolean ret = TRUE;
 #if PK_CHECK_VERSION(0,9,1)
-	PkResults *results;
-	GError *error_local = NULL;
+	_cleanup_error_free_ GError *error_local = NULL;
+	_cleanup_object_unref_ PkResults *results = NULL;
 
 	/* do sync call */
 	gs_plugin_status_update (plugin, NULL, GS_PLUGIN_STATUS_WAITING);
@@ -475,20 +480,13 @@ gs_plugin_app_source_remove (GsPlugin *plugin,
 		/* fall back to disabling it */
 		g_warning ("ignoring source remove error, trying disable: %s",
 			   error_local->message);
-		g_error_free (error_local);
-		ret = gs_plugin_app_source_disable (plugin,
-						    app,
-						    cancellable,
-						    error);
-		goto out;
+		return gs_plugin_app_source_disable (plugin, app,
+						     cancellable, error);
 	}
-out:
-	if (results != NULL)
-		g_object_unref (results);
+	return TRUE;
 #else
-	ret = gs_plugin_app_source_disable (plugin, app, cancellable, error);
+	return gs_plugin_app_source_disable (plugin, app, cancellable, error);
 #endif
-	return ret;
 }
 
 /**
@@ -501,37 +499,32 @@ gs_plugin_app_remove (GsPlugin *plugin,
 		      GError **error)
 {
 	const gchar *package_id;
-	gchar **package_ids = NULL;
-	gboolean ret = TRUE;
-	GPtrArray *array = NULL;
-	PkError *error_code = NULL;
-	PkResults *results = NULL;
 	GPtrArray *source_ids;
 	guint i;
 	guint cnt = 0;
+	_cleanup_object_unref_ PkError *error_code = NULL;
+	_cleanup_object_unref_ PkResults *results = NULL;
+	_cleanup_ptrarray_unref_ GPtrArray *array = NULL;
+	_cleanup_strv_free_ gchar **package_ids = NULL;
 
 	/* only process this app if was created by this plugin */
 	if (g_strcmp0 (gs_app_get_management_plugin (app), "PackageKit") != 0)
-		goto out;
+		return TRUE;
 
 	/* remove repo and all apps in it */
 	if (gs_app_get_kind (app) == GS_APP_KIND_SOURCE) {
-		ret = gs_plugin_app_source_remove (plugin,
-						   app,
-						   cancellable,
-						   error);
-		goto out;
+		return gs_plugin_app_source_remove (plugin, app,
+						    cancellable, error);
 	}
 
 	/* get the list of available package ids to install */
 	source_ids = gs_app_get_source_ids (app);
 	if (source_ids->len == 0) {
-		ret = FALSE;
 		g_set_error_literal (error,
 				     GS_PLUGIN_ERROR,
 				     GS_PLUGIN_ERROR_NOT_SUPPORTED,
 				     "removing not available");
-		goto out;
+		return FALSE;
 	}
 	package_ids = g_new0 (gchar *, source_ids->len + 1);
 	for (i = 0; i < source_ids->len; i++) {
@@ -541,12 +534,11 @@ gs_plugin_app_remove (GsPlugin *plugin,
 		package_ids[cnt++] = g_strdup (package_id);
 	}
 	if (cnt == 0) {
-		ret = FALSE;
 		g_set_error_literal (error,
 				     GS_PLUGIN_ERROR,
 				     GS_PLUGIN_ERROR_NOT_SUPPORTED,
 				     "no packages to remove");
-		goto out;
+		return FALSE;
 	}
 
 	/* do the action */
@@ -557,10 +549,8 @@ gs_plugin_app_remove (GsPlugin *plugin,
 						cancellable,
 						gs_plugin_packagekit_progress_cb, plugin,
 						error);
-	if (results == NULL) {
-		ret = FALSE;
-		goto out;
-	}
+	if (results == NULL)
+		return FALSE;
 
 	/* no longer valid */
 	gs_app_clear_source_ids (app);
@@ -568,22 +558,13 @@ gs_plugin_app_remove (GsPlugin *plugin,
 	/* check error code */
 	error_code = pk_results_get_error_code (results);
 	if (error_code != NULL) {
-		ret = FALSE;
 		g_set_error (error,
 			     GS_PLUGIN_ERROR,
 			     GS_PLUGIN_ERROR_FAILED,
 			     "failed to remove package: %s, %s",
 			     pk_error_enum_to_string (pk_error_get_code (error_code)),
 			     pk_error_get_details (error_code));
-		goto out;
+		return FALSE;
 	}
-out:
-	g_strfreev (package_ids);
-	if (error_code != NULL)
-		g_object_unref (error_code);
-	if (array != NULL)
-		g_ptr_array_unref (array);
-	if (results != NULL)
-		g_object_unref (results);
-	return ret;
+	return TRUE;
 }
